library(kernlab)

################ Exercise 1 ###################

credit<-read.csv('/Users/Thierry/Documents/R/Data/Economics/Greene/TableF7-3.csv',
                 sep=",",header=TRUE)

n<-nrow(credit)
ntrain<-10000
ntest<-n-ntrain
train<-sample(n,ntrain)

credit$CARDHLDR<-as.factor(credit$CARDHLDR)


#### CARDHLDR prediction problem

credit1<-credit[,-c(2,12:14)]
credit1.train<-credit1[train,]
credit1.test<-credit1[-train,]
y.test<-credit1.test$CARDHLDR

# Linear SVM

CC<-c(0.01,0.1,1,10,100,1000)
N<-length(CC)
err<-rep(0,N)
for(i in 1:N){
  print(i)
  err[i]<-cross(ksvm(CARDHLDR~.,data=credit1.train,scaled=TRUE,
                     kernel="vanilladot",C=CC[i],cross=5))
  }
plot(CC,err,type="b",log="x",xlab="C",ylab="CV error")

fit<-ksvm(CARDHLDR~.,data=credit1.train,scaled=TRUE,
          kernel="vanilladot",C=1)
pred<-predict(fit,newdata=credit1.test)
1-mean(y.test==pred)

# Nonlinear SVM

CC<-c(0.01,0.1,1,10)
N<-length(CC)
err<-rep(0,N)
for(i in 1:N){
  print(i)
  err[i]<-cross(ksvm(CARDHLDR~.,data=credit1.train,scaled=TRUE,
                     kernel="rbfdot",C=CC[i],cross=5))
}
plot(CC,err,type="b",log="x",xlab="C",ylab="CV error")

fit<-ksvm(CARDHLDR~.,data=credit1.train,scaled=TRUE,
          kernel="rbfdot",C=1)
pred<-predict(fit,newdata=credit1.test)
table(y.test,pred)
1-mean(y.test==pred)

# Logistic regression

fit.lr<-glm(CARDHLDR~.,data=credit1.train,family="binomial")
pred.lr=predict(fit.lr,newdata=credit1.test,type="response")
yhat3<-pred.lr>0.5
CM3<-table(y.test,yhat3)
1-sum(diag(CM3))/ntest

# Experiment with smaller datasets
ERR<-matrix(0,100,3)
for(i in 1:100){
  print(i)
  ii<-sample(ntrain,100)
  credit1.train.s<-credit1.train[ii,]
  
  fit<-ksvm(CARDHLDR~.,data=credit1.train.s,scaled=TRUE,
            kernel="vanilladot",C=1)
  pred<-predict(fit,newdata=credit1.test)
  ERR[i,1]<-1-mean(y.test==pred)
  
  fit<-ksvm(CARDHLDR~.,data=credit1.train.s,scaled=TRUE,
            kernel="rbfdot",C=0.1)
  pred<-predict(fit,newdata=credit1.test)
  ERR[i,2]<-1-mean(y.test==pred)
  
  fit.lr<-glm(CARDHLDR~.,data=credit1.train.s,family="binomial")
  pred.lr=predict(fit.lr,newdata=credit1.test,type="response")
  yhat3<-pred.lr>0.5
  CM3<-table(y.test,yhat3)
  ERR[i,3]<-1-sum(diag(CM3))/ntest
}
print(colMeans(ERR))
boxplot(ERR)


#### DEFAULT prediction problem

credit$DEFAULT<-as.factor(credit$DEFAULT)

ii<-which(credit$CARDHLDR==1)
credit2<-credit[ii,-1]
n<-nrow(credit2)
ntrain<-5000
ntest<-n-ntrain
train<-sample(n,ntrain)

credit2.train<-credit2[train,]
credit2.test<-credit2[-train,]
y.test<-credit2.test$DEFAULT

# Linear SVM

fit<-ksvm(DEFAULT~.,data=credit2.train,scaled=TRUE,
          kernel="vanilladot",C=1)
pred<-predict(fit,newdata=credit2.test)
1-mean(y.test==pred)

# Nonlinear SVM

fit<-ksvm(DEFAULT~.,data=credit2.train,scaled=TRUE,
          kernel="rbfdot",C=1)
pred<-predict(fit,newdata=credit2.test)
1-mean(y.test==pred)

# Logistic regression
fit.lr<-glm(DEFAULT~.,data=credit2.train,family="binomial")
pred.lr=predict(fit.lr,newdata=credit2.test,type="response")
yhat3<-pred.lr>0.5
CM3<-table(y.test,yhat3)
1-sum(diag(CM3))/ntest


################ Exercise 2 ###################

# Q1
library(MASS)  
n<-nrow(Boston)


ntrain<-round(2*n/3)
ntest<-n-ntrain
train<-sample(n,ntrain)
Boston.train<-Boston[train,]
Boston.test<-Boston[-train,]
y.test<-Boston.test$medv


# Linear SVM
fit<-ksvm(medv~.,data=Boston.train,scaled=TRUE,
          kernel="vanilladot",C=10)
pred.svr1<-predict(fit,newdata=Boston.test)
mse.svr1<-mean((y.test-pred.svr1)^2)
print(mse.svr1)
plot(y.test,pred.svr1,col="blue",pch=1)

# Nonlinear SVM
fit<-ksvm(medv~.,data=Boston.train,scaled=TRUE,
          kernel="rbfdot",C=10)
pred.svr2<-predict(fit,newdata=Boston.test)
mse.svr2<-mean((y.test-pred.svr2)^2)
print(mse.svr2)
points(y.test,pred.svr2,col="red",pch=1)

# Comparison with LS linear regression
reg<- lm(medv ~. ,data=Boston.train)
pred<-predict(reg,newdata=Boston.test)
mse.ls<-mean((y.test-pred)^2)
print(mse.ls)
points(y.test,pred,col="green",pch=3)


# Comparison with random forest
library(randomForest)
fit.rf<-randomForest(medv~.,data=Boston.train)
pred.rf<-predict(fit.rf,newdata=Boston.test)
mse.rf<-mean((y.test-pred.rf)^2)
print(mse.rf)
points(y.test,pred.rf,col="red",pch=2)

# Linear RVM

fit.rvm1<-rvm(medv~.,data=Boston.train,type="regression",
         kernel="vanilladot")
pred.rvm1<-predict(fit.rvm1,newdata=Boston.test)
mse.rvm1<-mean((y.test-pred.rvm1)^2)
print(mse.rvm1)
plot(y.test,pred.rvm1)


# Nonlinear RVM

fit.rvm2<-rvm(medv~.-chas,data=Boston.train,
              kernel="laplacedot",kpar="automatic")
pred.rvm2<-predict(fit.rvm2,newdata=Boston.test)
mse.rvm2<-mean((y.test-pred.rvm2)^2)
print(mse.rvm2)
plot(y.test,pred.rvm2)


#Q2
movie<-read.csv('/Users/Thierry/Documents/R/Data/Economics/Greene/movie_buzz.csv',
                header=TRUE,sep=",")

# Preprocessing

# declaring the factors
movie$MPRATING<-as.factor(movie$MPRATING)
levels(movie$MPRATING)<-c("G","PG","PG13","R")
movie$SEQUEL<-as.factor(movie$SEQUEL)
movie$ACTION<-as.factor(movie$ACTION)
movie$COMEDY<-as.factor(movie$COMEDY)
movie$ANIMATED<-as.factor(movie$ANIMATED)
movie$HORROR<-as.factor(movie$HORROR)
# Log transformation
movie$BOX<-log(movie$BOX)
movie$BUDGET<-log(movie$BUDGET)

X<-model.matrix(BOX~.,data=movie)

n<-nrow(movie)
ntrain<-40
ntest<-n-ntrain
train<-sample(n,ntrain)
movie.train<-movie[train,]
movie.test<-movie[-train,]
X.train<-X[train,]
X.test<-X[-train,]
y.test<-movie.test$BOX

fit.rvm1<-rvm(BOX~.,data=movie.train,kernel="rbfdot")
pred.rvm1<-predict(fit.rvm1,newdata=movie.test)
mse.rvm1<-mean((y.test-pred.rvm1)^2)
print(mse.rvm1)
plot(y.test,pred.rvm1)



